package com.netmera.netmera_flutter_sdk;

import android.content.Context;
import android.content.Intent;
import android.content.res.AssetManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.work.Data;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkManager;
import androidx.work.Worker;
import androidx.work.WorkerParameters;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

import io.flutter.FlutterInjector;
import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.dart.DartExecutor;
import io.flutter.embedding.engine.loader.FlutterLoader;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.view.FlutterCallbackInformation;


/**
 * Copyright (c) 2021 Inomera Research.
 *
 * @author Burak Soykal
 */
public class FNetmeraWorker extends Worker implements MethodChannel.MethodCallHandler {

    private static final String INTENT_KEY = "INTENT_KEY";
    private static final String SERVICE_INITIALIZED = "serviceInitialized";
    private static final AtomicBoolean serviceStarted = new AtomicBoolean(false);
    private static final ArrayDeque<List<Object>> queue = new ArrayDeque<>();

    private static MethodChannel backgroundChannel;
    private static FlutterEngine flutterEngine;
    private static FlutterLoader flutterLoader;

    private final Context context;

    public FNetmeraWorker(@NonNull Context context, @NonNull WorkerParameters workerParams) {
        super(context, workerParams);
        this.context = context;
        flutterLoader = FlutterInjector.instance().flutterLoader();
    }

    public static void createAndStart(Context context, Intent intent) {
        OneTimeWorkRequest.Builder requestBuilder = new OneTimeWorkRequest.Builder(FNetmeraWorker.class);
        if (intent != null) {
            Data inputData = new Data.Builder()
                    .putAll(bundleToHashMap(intent.getExtras()))
                    .putString(INTENT_KEY, intent.getAction())
                    .build();
            requestBuilder.setInputData(inputData);
        }
        WorkManager.getInstance(context).enqueue(requestBuilder.build());
    }

    private void startService(final Context context) {

        flutterEngine = new FlutterEngine(context);

        if (!flutterLoader.initialized()) {
            flutterLoader.startInitialization(context);
        }

        Handler callbackHandler = new Handler(Looper.getMainLooper());
        Runnable callback = new Runnable() {
            @Override
            public void run() {
                Long callbackHandle = getFromSharedPref(context, NetmeraFlutterSdkPlugin.CALLBACK_DISPATCHER_HANDLE_KEY);
                FlutterCallbackInformation callbackInfo = FlutterCallbackInformation.lookupCallbackInformation(callbackHandle);
                String pathToBundle = flutterLoader.findAppBundlePath();

                if (flutterEngine != null && callbackInfo != null) {
                    backgroundChannel = new MethodChannel(flutterEngine.getDartExecutor(), "flutter_nm_sdk_background");
                    backgroundChannel.setMethodCallHandler(FNetmeraWorker.this);
                    AssetManager assetManager = getApplicationContext().getAssets();
                    DartExecutor.DartCallback dartCallback = new DartExecutor.DartCallback(assetManager, pathToBundle, callbackInfo);
                    flutterEngine.getDartExecutor().executeDartCallback(dartCallback);
                }
            }
        };

        flutterLoader.ensureInitializationCompleteAsync(context, null, callbackHandler, callback);
    }

    @Override
    public void onStopped() {
        super.onStopped();
        stopEngine();
    }

    private void stopEngine() {
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (flutterEngine != null) {
                    flutterEngine.destroy();
                    flutterEngine = null;
                }
            }
        });
    }

    private void invokeDart(List<Object> list) {
        if (backgroundChannel == null || list == null || list.size() == 0 || list.get(0) == null) {
            return;
        }
        Long callbackMethodRef = getFromSharedPref(context, (String) list.get(0));
        if (callbackMethodRef == 0) {
            return;
        }
        list.set(0, callbackMethodRef);
        backgroundChannel.invokeMethod("", list);
    }

    @NonNull
    @Override
    public Result doWork() {
        new Handler(Looper.getMainLooper()).post(
                new Runnable() {
                    @Override
                    public void run() {
                        if (getInputData().getKeyValueMap().size() == 0) {
                            if (!queue.isEmpty()) {
                                startService(context);
                            }
                            return;
                        }
                        final String action = getInputData().getString(INTENT_KEY);
                        final HashMap<String, Object> bundle = new HashMap<>();

                        for (Map.Entry<String, Object> entry : getInputData().getKeyValueMap().entrySet()) {
                            if (!entry.getKey().equals(INTENT_KEY)) {
                                bundle.put(entry.getKey(), entry.getValue());
                            }
                        }

                        if (flutterEngine == null || flutterLoader == null || backgroundChannel == null) {
                            startService(context);
                        }
                        ArrayList<Object> list = new ArrayList<>();
                        list.add(action);
                        list.add(bundle);
                        if (!serviceStarted.get()) {
                            queue.add(list);
                        } else {
                            invokeDart(list);
                        }
                    }
                });
        return Result.success();
    }

    @Override
    public void onMethodCall(@NonNull final MethodCall call, @NonNull final MethodChannel.Result result) {
        new Handler(Looper.getMainLooper()).post(
                new Runnable() {
                    @Override
                    public void run() {
                        if (SERVICE_INITIALIZED.equals(call.method)) {
                            while (!queue.isEmpty()) {
                                invokeDart(queue.remove());
                            }
                            serviceStarted.set(true);
                        }
                        result.success(null);
                    }
                });
    }

    private static Long getFromSharedPref(Context context, String key) {
        return context.getSharedPreferences(
                NetmeraFlutterSdkPlugin.SHARED_PREFERENCES_KEY,
                Context.MODE_PRIVATE)
                .getLong(key, 0);
    }

    private static HashMap<String, Object> bundleToHashMap(@Nullable Bundle bundle) {
        HashMap<String, Object> map = new HashMap<>();
        if (bundle == null) {
            return map;
        }
        Set<String> keys = bundle.keySet();
        for (String key : keys) {
            map.put(key, bundle.get(key));
        }
        return map;
    }
}
