/**
 * Copyright (c) 2019 Inomera Research.
 * @author Burak Soykal
 */

#import "FNetmeraService.h"

@implementation FNetmeraService

static NSUserDefaults *_persistentState;
static FlutterEngine *_headlessRunner;
static NSObject<FlutterPluginRegistrar>* _registrar = nil;
static FlutterPluginRegistrantCallback _registerPlugin = nil;
static FlutterMethodChannel *_callbackChannel;
static BOOL initialized = NO;
static NSMutableArray *_messageQueue;

NSString *const CALLBACK_DISPATCHER_HANDLE_KEY = @"callback_dispatch_handler";
NSString *const ON_PUSH_REGISTER = @"onPushRegister";
NSString *const ON_PUSH_RECEIVE = @"onPushReceive";
NSString *const ON_PUSH_OPEN = @"onPushOpen";
NSString *const ON_PUSH_DISMISS = @"onPushDismiss";
NSString *const ON_PUSH_BUTTON_CLICKED = @"onPushButtonClicked";

+ (void)initService:(FlutterMethodCall *)call registerPlugin: (FlutterPluginRegistrantCallback) registerPlugin registrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    _registrar = registrar;
    _registerPlugin = registerPlugin;
    _persistentState = [NSUserDefaults standardUserDefaults];
    if (_messageQueue == nil) {
        _messageQueue = [[NSMutableArray alloc] init];
    }
    _headlessRunner = [[FlutterEngine alloc] initWithName:@"GeofencingIsolate" project:nil allowHeadlessExecution:YES];
    _callbackChannel = [FlutterMethodChannel methodChannelWithName:@"flutter_nm_sdk_background" binaryMessenger:_headlessRunner];
    
    [self setCallbackDispatcherHandle:CALLBACK_DISPATCHER_HANDLE_KEY handle: [[((NSMutableDictionary *)call.arguments) objectForKey:(CALLBACK_DISPATCHER_HANDLE_KEY)] longLongValue]];
    [self setCallbackDispatcherHandle:ON_PUSH_REGISTER handle:[[((NSMutableDictionary *)call.arguments) objectForKey:(ON_PUSH_REGISTER)] longLongValue]];
    [self setCallbackDispatcherHandle:ON_PUSH_RECEIVE handle:[[((NSMutableDictionary *)call.arguments) objectForKey:(ON_PUSH_RECEIVE)] longLongValue]];
    [self setCallbackDispatcherHandle:ON_PUSH_DISMISS handle:[[((NSMutableDictionary *)call.arguments) objectForKey:(ON_PUSH_DISMISS)] longLongValue]];
    [self setCallbackDispatcherHandle:ON_PUSH_OPEN handle:[[((NSMutableDictionary *)call.arguments) objectForKey:(ON_PUSH_OPEN)] longLongValue]];
    [self setCallbackDispatcherHandle:ON_PUSH_BUTTON_CLICKED handle:[[((NSMutableDictionary *)call.arguments) objectForKey:(ON_PUSH_BUTTON_CLICKED)] longLongValue]];
    
    [self startService];
}
+ (void)startService {
    int64_t callbackHandle = [self getCallbackDispatcherHandle:CALLBACK_DISPATCHER_HANDLE_KEY];
    FlutterCallbackInformation *info = [FlutterCallbackCache lookupCallbackInformation:callbackHandle];
    
    if(info==nil || _registerPlugin==nil) return;
   
    NSString *entrypoint = info.callbackName;
    NSString *uri = info.callbackLibraryPath;
    [_headlessRunner runWithEntrypoint:entrypoint libraryURI:uri];
    _registerPlugin(_headlessRunner);
    [_registrar addMethodCallDelegate:self channel:_callbackChannel];
}

+ (void)setCallbackDispatcherHandle:(NSString*) key handle: (int64_t)handle {
    [_persistentState setObject:[NSNumber numberWithLongLong:handle] forKey:key];
}

+ (int64_t)getCallbackDispatcherHandle: (NSString*) key {
    id handle = [_persistentState objectForKey:key];
    if (handle == nil) {
        return 0;
    }
    return [handle longLongValue];
}

+ (void)sendMessage:(NSDictionary*)message {
    int64_t handle = [self getCallbackDispatcherHandle:message[@"action"]];
    NSArray* argumentList = @[@(handle),message[@"message"]];
    
    [_callbackChannel
     invokeMethod:@""
     arguments:argumentList];
}

+ (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    @synchronized(self) {
        initialized = YES;
        while ([_messageQueue count] > 0) {
            NSDictionary* message = _messageQueue[0];
            [_messageQueue removeObjectAtIndex:0];
            [self sendMessage: message];
        }
    }
}

+ (void) handleWork: (NSString*)action dict:(NSDictionary*)dict {
    @synchronized(self) {
        NSDictionary *message = @{
                               @"action": action,
                               @"message": dict
                               };
        if (initialized) {
            [self sendMessage:message];
        } else {
            if (_messageQueue == nil) {
                _messageQueue = [[NSMutableArray alloc] init];
            }
            [_messageQueue addObject:message];
        }
    }
}

@end
